"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.colorizeTokens = void 0;
const trivia_1 = require("./trivia");
const line_composer_1 = require("./line-composer");
const theme_wrapper_1 = require("../themes/theme-wrapper");
const chalk_1 = __importDefault(require("chalk"));
async function colorizeTokens(source, width, { themeIdOrPath, languageIdOrPath }) {
    const lines = trivia_1.splitLines(source);
    const theme = await theme_wrapper_1.ThemeWrapper.create(themeIdOrPath);
    const grammarWrapper = await theme.loadGrammar(languageIdOrPath);
    const tokens = grammarWrapper.tokenizeLines(lines);
    return lines.map((line, lineIndex) => {
        const composer = new line_composer_1.LineComposer(width, theme);
        let lastCharTaken = 0;
        for (const { startColumn, endColumn, foregroundColor } of tokens[lineIndex]) {
            if (startColumn > lastCharTaken) {
                composer.append(line.substring(lastCharTaken, startColumn));
            }
            const brush = foregroundColor ? chalk_1.default.hex(foregroundColor) : undefined;
            composer.append(line.substring(startColumn, endColumn), brush);
            lastCharTaken = endColumn;
        }
        if (lastCharTaken < line.length) {
            composer.append(line.substring(lastCharTaken));
        }
        return composer.compose();
    });
}
exports.colorizeTokens = colorizeTokens;
//# sourceMappingURL=colorize-tokens.js.map