"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.highlightDiagnostic = void 0;
const trivia_1 = require("./trivia");
const line_composer_1 = require("./line-composer");
const options_1 = require("../options");
const theme_wrapper_1 = require("../themes/theme-wrapper");
const colorize_tokens_1 = require("./colorize-tokens");
const trivia_2 = require("./trivia");
async function highlightDiagnostic(options) {
    const { source, range: { from, to }, severity, themeIdOrPath, width, } = options;
    const marginWidth = trivia_1.calculateMarginWidth(source, to.line - from.line + 1);
    const sourceWidth = width - marginWidth - trivia_2.MARGIN_MIDDLE.length - 1;
    const theme = await theme_wrapper_1.ThemeWrapper.create(themeIdOrPath);
    const marginBrush = theme.foreground("editorLineNumber.foreground");
    const diagnosticBrush = getDiagnosticColor(theme, severity);
    const colorized = await colorize_tokens_1.colorizeTokens(source, sourceWidth, options);
    const highlighted = colorized.map((line, lineIndex) => {
        const lineNumber = (lineIndex + 1).toString().padStart(marginWidth, " ");
        return marginBrush(lineNumber + trivia_2.MARGIN_MIDDLE) + line;
    });
    let lineIndex = from.line;
    let i = from.line;
    const originalLines = trivia_1.splitLines(source);
    for (; lineIndex <= to.line; lineIndex++, i += 2) {
        const startColumn = lineIndex === from.line ? from.column : 0;
        const endColumn = lineIndex === to.line ? to.column : originalLines[lineIndex].length;
        const composer = new line_composer_1.LineComposer(sourceWidth, theme, /* renderWhitespace */ false);
        composer.append(" ".repeat(startColumn));
        composer.append(trivia_2.SQUIGGLY_MARKER.repeat(endColumn - startColumn), diagnosticBrush);
        highlighted.splice(i + 1, 0, " ".repeat(marginWidth) + marginBrush(trivia_2.MARGIN_MIDDLE) + composer.compose());
    }
    const startLine = Math.max(0, from.line - trivia_2.CONTEXT_LINES);
    const endLine = Math.min(highlighted.length, i + trivia_2.CONTEXT_LINES);
    const backgroundBrush = theme.background("editor.background");
    return highlighted.slice(startLine, endLine).map((line) => backgroundBrush(" " + line));
}
exports.highlightDiagnostic = highlightDiagnostic;
function getDiagnosticColor(theme, severity) {
    switch (severity) {
        case options_1.DiagnosticSeverity.Error:
            return theme.foreground("editorError.foreground");
        case options_1.DiagnosticSeverity.Warning:
            return theme.foreground("editorWarning.foreground");
        case options_1.DiagnosticSeverity.Info:
            return theme.foreground("editorInfo.foreground");
        case options_1.DiagnosticSeverity.Hint:
            return theme.foreground("editorHint.foreground");
    }
}
//# sourceMappingURL=highlight-diagnostics.js.map