"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.highlightDiff = void 0;
const trivia_1 = require("./trivia");
const diff = __importStar(require("diff"));
const colorize_tokens_1 = require("./colorize-tokens");
const theme_wrapper_1 = require("../themes/theme-wrapper");
const trivia_2 = require("./trivia");
async function highlightDiff(options) {
    const { beforeSource, afterSource, themeIdOrPath, width } = options;
    const chunks = computeDifferences(beforeSource, afterSource);
    if (chunks.length === 1 && chunks[0].type === "same") {
        // No changes
        return [];
    }
    const theme = await theme_wrapper_1.ThemeWrapper.create(themeIdOrPath);
    const marginBrush = theme.foreground("editorLineNumber.foreground");
    const insertedBrush = theme.background("diffEditor.insertedTextBackground");
    const removedBrush = theme.background("diffEditor.removedTextBackground");
    const widestMargin = Math.max(trivia_1.calculateMarginWidth(beforeSource), trivia_1.calculateMarginWidth(afterSource));
    const sourceWidth = width - (widestMargin + trivia_2.MARGIN_MIDDLE.length) * 2 - 1;
    const beforeColorized = await colorize_tokens_1.colorizeTokens(beforeSource, sourceWidth, options);
    const afterColorized = await colorize_tokens_1.colorizeTokens(afterSource, sourceWidth, options);
    const renderHiddenRangeLine = (arrow) => marginBrush((" ".repeat(widestMargin) + arrow).repeat(2)) + " ".repeat(sourceWidth);
    const result = [];
    chunks.forEach(({ type, lines }, chunkIndex, chunks) => {
        function renderLine({ beforeLineNumber, afterLineNumber }) {
            switch (type) {
                case "inserted":
                    return (marginBrush(" ".repeat(widestMargin) + trivia_2.MARGIN_MIDDLE) +
                        marginBrush((afterLineNumber + 1).toString().padStart(widestMargin) + trivia_2.MARGIN_MIDDLE) +
                        insertedBrush(afterColorized[afterLineNumber]));
                case "removed":
                    return (marginBrush((beforeLineNumber + 1).toString().padStart(widestMargin) + trivia_2.MARGIN_MIDDLE) +
                        marginBrush(" ".repeat(widestMargin) + trivia_2.MARGIN_MIDDLE) +
                        removedBrush(beforeColorized[beforeLineNumber]));
                case "same":
                    return (marginBrush((beforeLineNumber + 1).toString().padStart(widestMargin) + trivia_2.MARGIN_MIDDLE) +
                        marginBrush((afterLineNumber + 1).toString().padStart(widestMargin) + trivia_2.MARGIN_MIDDLE) +
                        beforeColorized[beforeLineNumber]);
            }
        }
        if (type === "same") {
            if (chunkIndex === 0 && lines.length > trivia_2.CONTEXT_LINES + 1) {
                result.push(renderHiddenRangeLine(trivia_2.MARGIN_UP), ...lines.slice(lines.length - trivia_2.CONTEXT_LINES).map(renderLine));
                return;
            }
            if (chunkIndex === chunks.length - 1 && lines.length > trivia_2.CONTEXT_LINES + 1) {
                result.push(...lines.slice(0, trivia_2.CONTEXT_LINES).map(renderLine), renderHiddenRangeLine(trivia_2.MARGIN_DOWN));
                return;
            }
            if (lines.length > trivia_2.CONTEXT_LINES * 2 + 2) {
                result.push(...lines.slice(0, trivia_2.CONTEXT_LINES).map(renderLine), renderHiddenRangeLine(trivia_2.MARGIN_DOWN), renderHiddenRangeLine(trivia_2.MARGIN_UP), ...lines.slice(lines.length - trivia_2.CONTEXT_LINES).map(renderLine));
                return;
            }
        }
        result.push(...lines.map(renderLine));
    });
    const backgroundBrush = theme.background("editor.background");
    return result.map((line) => backgroundBrush(" " + line));
}
exports.highlightDiff = highlightDiff;
function computeDifferences(before, after) {
    let beforeLineCounter = 0;
    let afterLineCounter = 0;
    return diff.diffLines(before, after).map((change) => {
        const lines = trivia_1.splitLines(change.value.replace(/\r?\n$/, "")).map((_, delta) => ({
            beforeLineNumber: beforeLineCounter + delta,
            afterLineNumber: afterLineCounter + delta,
        }));
        let type;
        if (change.added) {
            type = "inserted";
            afterLineCounter += lines.length;
        }
        else if (change.removed) {
            type = "removed";
            beforeLineCounter += lines.length;
        }
        else {
            type = "same";
            afterLineCounter += lines.length;
            beforeLineCounter += lines.length;
        }
        return { type, lines };
    });
}
//# sourceMappingURL=highlight-diffs.js.map