"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.highlightSource = void 0;
const colorize_tokens_1 = require("./colorize-tokens");
const trivia_1 = require("./trivia");
const theme_wrapper_1 = require("../themes/theme-wrapper");
async function highlightSource(options) {
    const { source, themeIdOrPath, width } = options;
    const marginWidth = trivia_1.calculateMarginWidth(source);
    const sourceWidth = width - marginWidth - trivia_1.MARGIN_MIDDLE.length - 1;
    const colorized = await colorize_tokens_1.colorizeTokens(source, sourceWidth, options);
    const theme = await theme_wrapper_1.ThemeWrapper.create(themeIdOrPath);
    const marginBrush = theme.foreground("editorLineNumber.foreground");
    const backgroundBrush = theme.background("editor.background");
    return colorized.map((line, lineIndex) => {
        const lineNumber = (lineIndex + 1).toString().padStart(marginWidth, " ");
        return backgroundBrush(" " + marginBrush(lineNumber + trivia_1.MARGIN_MIDDLE) + line);
    });
}
exports.highlightSource = highlightSource;
//# sourceMappingURL=highlight-source.js.map