"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LineComposer = void 0;
const trivia_1 = require("./trivia");
class LineComposer {
    constructor(width, theme, renderWhitespace = true) {
        this.width = width;
        this.theme = theme;
        this.renderWhitespace = renderWhitespace;
        this.parts = [];
    }
    append(part, brush = this.theme.foreground("editor.foreground")) {
        if (part.length === 0) {
            return;
        }
        if (!this.renderWhitespace) {
            this.parts.push([part, brush]);
            return;
        }
        this.processWhitespace(part, / /g, trivia_1.SPACE_REPLACEMENT, (betweenSpaces) => {
            this.processWhitespace(betweenSpaces, /\t/g, trivia_1.TAB_REPLACEMENT, (betweenTabs) => {
                this.parts.push([betweenTabs, brush]);
            });
        });
    }
    compose() {
        let remainingWidth = this.width;
        const coloredParts = [];
        for (let i = 0; i < this.parts.length; i++) {
            const [value, brush] = this.parts[i];
            // Whole part fits, just add it:
            if (value.length < remainingWidth) {
                coloredParts.push(brush(value));
                remainingWidth -= value.length;
                continue;
            }
            // Whole part fits, but nothing more would be added.
            if (value.length === remainingWidth) {
                if (i + 1 === this.parts.length) {
                    // This is the last part. Add as-is.
                    coloredParts.push(brush(value));
                }
                else {
                    // There are more parts, add elepsis and skip the rest
                    const clippedValue = value.substr(0, value.length - trivia_1.ELEPSIS.length) + trivia_1.ELEPSIS;
                    coloredParts.push(brush(clippedValue));
                }
            }
            else {
                // Only a substring of the part can fit
                const clippedValue = value.substr(0, remainingWidth - trivia_1.ELEPSIS.length) + trivia_1.ELEPSIS;
                coloredParts.push(brush(clippedValue));
            }
            remainingWidth = 0;
            break;
        }
        return coloredParts.join("") + " ".repeat(remainingWidth);
    }
    processWhitespace(value, separator, replacement, next) {
        const parts = value.split(separator);
        if (parts.length === 0) {
            return;
        }
        next(parts[0]);
        const whitespaceBrush = this.theme.foreground("editorWhitespace.foreground");
        for (let i = 1; i < parts.length; i++) {
            this.parts.push([replacement, whitespaceBrush]);
            next(parts[i]);
        }
    }
}
exports.LineComposer = LineComposer;
//# sourceMappingURL=line-composer.js.map