"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getSupportedThemes = exports.getSupportedLanguages = exports.highlightDiff = exports.highlightDiagnostic = exports.highlightSource = void 0;
const joi_1 = __importDefault(require("joi"));
const highlight_source_1 = require("./highlighting/highlight-source");
const highlight_diagnostics_1 = require("./highlighting/highlight-diagnostics");
const highlight_diffs_1 = require("./highlighting/highlight-diffs");
const metadata_1 = require("./languages/metadata");
const metadata_2 = require("./themes/metadata");
const options_1 = require("./options");
async function highlightSource(options) {
    const schema = await options_1.HighlightSourceOptionsSchema();
    const validatedOptions = joi_1.default.attempt(options, schema);
    return highlight_source_1.highlightSource(validatedOptions);
}
exports.highlightSource = highlightSource;
async function highlightDiagnostic(options) {
    const schema = await options_1.HighlightDiagnosticOptionsSchema();
    const validatedOptions = joi_1.default.attempt(options, schema);
    return highlight_diagnostics_1.highlightDiagnostic(validatedOptions);
}
exports.highlightDiagnostic = highlightDiagnostic;
async function highlightDiff(options) {
    const schema = await options_1.HighlightDiffOptionsSchema();
    const validatedOptions = joi_1.default.attempt(options, schema);
    return highlight_diffs_1.highlightDiff(validatedOptions);
}
exports.highlightDiff = highlightDiff;
async function getSupportedLanguages() {
    const languages = await metadata_1.getSupportedLanguages();
    return languages.map((language) => ({
        id: language.id,
    }));
}
exports.getSupportedLanguages = getSupportedLanguages;
async function getSupportedThemes() {
    const themes = await metadata_2.getSupportedThemes();
    return themes.map((theme) => ({
        id: theme.id,
    }));
}
exports.getSupportedThemes = getSupportedThemes;
//# sourceMappingURL=index.js.map