"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GrammarWrapper = void 0;
class GrammarWrapper {
    constructor(rawGrammar, colorMap) {
        this.rawGrammar = rawGrammar;
        this.colorMap = colorMap;
    }
    tokenizeLines(lines) {
        let state = null;
        return lines.map((line, lineIndex) => {
            const { tokens, nextState } = this.tokenizeSingleLine(line, lineIndex, state);
            state = nextState;
            return tokens;
        });
    }
    tokenizeSingleLine(line, lineIndex, previousState) {
        const { tokens: oldTokens, ruleStack: nextState } = this.rawGrammar.tokenizeLine2(line, previousState);
        const tokensCount = oldTokens.length / 2;
        const newTokens = [];
        for (let j = 0; j < tokensCount; j++) {
            const startColumn = oldTokens[2 * j];
            const endColumn = j + 1 < tokensCount ? oldTokens[2 * j + 2] : line.length;
            if (startColumn == endColumn) {
                // Empty recognition
                continue;
            }
            const metadata = oldTokens[2 * j + 1];
            const foregroundColor = this.colorMap[(metadata & 8372224 /* FOREGROUND_MASK */) >>> 14 /* FOREGROUND_OFFSET */];
            if (foregroundColor !== "#000000") {
                newTokens.push({
                    startColumn,
                    endColumn,
                    line: lineIndex,
                    foregroundColor,
                });
                continue;
            }
            let running = "";
            for (let i = startColumn; i < endColumn; i++) {
                const char = line[i];
                if (/ |\t/.test(char)) {
                    if (running.length > 0) {
                        newTokens.push({
                            startColumn: i - running.length,
                            endColumn: i,
                            line: lineIndex,
                        });
                        running = "";
                    }
                }
                else {
                    running += char;
                }
            }
            if (running.length > 0) {
                newTokens.push({
                    startColumn: endColumn - running.length,
                    endColumn,
                    line: lineIndex,
                });
            }
        }
        return { tokens: newTokens, nextState };
    }
}
exports.GrammarWrapper = GrammarWrapper;
//# sourceMappingURL=grammar-wrapper.js.map