"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.detectLanguage = void 0;
const path_1 = __importDefault(require("path"));
const micromatch_1 = __importDefault(require("micromatch"));
const metadata_1 = require("./metadata");
const trivia_1 = require("../highlighting/trivia");
async function detectLanguage(filePath, contents) {
    if (!path_1.default.isAbsolute(filePath)) {
        filePath = path_1.default.resolve(filePath);
    }
    const languages = await metadata_1.getSupportedLanguages();
    for (const { id, extensions, fileNamePatterns, firstLines, fileNames } of languages) {
        if (extensions.includes(path_1.default.extname(filePath))) {
            return id;
        }
        if (fileNames.includes(path_1.default.basename(filePath))) {
            return id;
        }
        if ([true, false].some((matchBase) => {
            return micromatch_1.default.isMatch(filePath, fileNamePatterns, { matchBase });
        })) {
            return id;
        }
        const firstLine = trivia_1.splitLines(contents)[0];
        if (firstLines.some((pattern) => new RegExp(pattern).test(firstLine))) {
            return id;
        }
    }
    return undefined;
}
exports.detectLanguage = detectLanguage;
//# sourceMappingURL=language-detection.js.map