"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HighlightDiffOptionsSchema = exports.HighlightDiagnosticOptionsSchema = exports.DiagnosticSeverity = exports.HighlightSourceOptionsSchema = exports.BaseHighlightOptionsSchema = void 0;
const joi_1 = __importDefault(require("joi"));
async function BaseHighlightOptionsSchema() {
    return joi_1.default
        .object({
        themeIdOrPath: joi_1.default.string(),
        languageIdOrPath: joi_1.default.string(),
        width: joi_1.default.number().required().min(10).max(1000),
    })
        .required();
}
exports.BaseHighlightOptionsSchema = BaseHighlightOptionsSchema;
async function HighlightSourceOptionsSchema() {
    const baseSchema = await BaseHighlightOptionsSchema();
    return baseSchema.keys({
        source: joi_1.default.string().required().min(1),
    });
}
exports.HighlightSourceOptionsSchema = HighlightSourceOptionsSchema;
var DiagnosticSeverity;
(function (DiagnosticSeverity) {
    DiagnosticSeverity["Error"] = "error";
    DiagnosticSeverity["Warning"] = "warning";
    DiagnosticSeverity["Info"] = "info";
    DiagnosticSeverity["Hint"] = "hint";
})(DiagnosticSeverity = exports.DiagnosticSeverity || (exports.DiagnosticSeverity = {}));
async function HighlightDiagnosticOptionsSchema() {
    const baseSchema = await BaseHighlightOptionsSchema();
    return baseSchema.keys({
        source: joi_1.default.string().required().min(1),
        severity: joi_1.default.string().required().valid("error", "warning", "info", "hint"),
        range: joi_1.default
            .object({
            from: joi_1.default
                .object({
                line: joi_1.default.number().required().min(0).max(10000),
                column: joi_1.default.number().required().min(0).max(10000),
            })
                .required(),
            to: joi_1.default
                .object({
                line: joi_1.default.number().required().min(0).max(10000),
                column: joi_1.default.number().required().min(0).max(10000),
            })
                .required(),
        })
            .required(),
    });
}
exports.HighlightDiagnosticOptionsSchema = HighlightDiagnosticOptionsSchema;
async function HighlightDiffOptionsSchema() {
    const baseSchema = await BaseHighlightOptionsSchema();
    return baseSchema.keys({
        beforeSource: joi_1.default.string().required().min(1),
        afterSource: joi_1.default.string().required().min(1),
    });
}
exports.HighlightDiffOptionsSchema = HighlightDiffOptionsSchema;
//# sourceMappingURL=options.js.map