"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getOnigLib = void 0;
// Using "vscode-oniguruma" is not possible, as it depends on vscode-specific libraries that are not cleanly shared
// Using the original "oniguruma" causes issues with jest module loader, as it overrides symbols imported from the native module.
// Overrides here ./node_modules/oniguruma/src/oniguruma.js
// If the module is loaded twice, the second load fails with "TypeError: Cannot redefine property: length".
// Therefore, we have to implement a better approach to this, by adding the overrides to a child type instead.
const { OnigScanner: OriginalOnigScanner, OnigString: OriginalOnigString, } = require("oniguruma/build/Release/onig_scanner.node");
class CustomOnigScanner extends OriginalOnigScanner {
    constructor(patterns) {
        super(patterns);
    }
    findNextMatch(string, startPosition, callback) {
        if (startPosition == null)
            startPosition = 0;
        if (typeof startPosition === "function") {
            callback = startPosition;
            startPosition = 0;
        }
        string = this.convertToString(string);
        startPosition = this.convertToNumber(startPosition);
        this._findNextMatch(string, startPosition, (error, match) => {
            if (match)
                match.scanner = this;
            return callback(error, match);
        });
    }
    findNextMatchSync(string, startPosition) {
        if (startPosition == null) {
            startPosition = 0;
        }
        string = this.convertToString(string);
        startPosition = this.convertToNumber(startPosition);
        let match = this._findNextMatchSync(string, startPosition);
        if (match)
            match.scanner = this;
        return match;
    }
    convertToString(value) {
        if (value === undefined)
            return "undefined";
        if (value === null)
            return "null";
        if (value.constructor == OnigString)
            return value;
        return value.toString();
    }
    convertToNumber(value) {
        value = parseInt(value);
        if (!isFinite(value)) {
            value = 0;
        }
        value = Math.max(value, 0);
        return value;
    }
}
class OnigString extends OriginalOnigString {
    constructor(content) {
        super(content);
    }
    substring(start, end) {
        return this.content.substring(start, end);
    }
    toString() {
        return this.content;
    }
    get length() {
        return this.content.length;
    }
}
function getOnigLib() {
    return Promise.resolve({
        createOnigScanner: (patterns) => new CustomOnigScanner(patterns),
        createOnigString: (content) => new OnigString(content),
    });
}
exports.getOnigLib = getOnigLib;
//# sourceMappingURL=oniguruma-loader.js.map