"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveTheme = void 0;
const assert_1 = __importDefault(require("assert"));
const fs_extra_1 = require("fs-extra");
const JSONC = __importStar(require("jsonc-parser"));
const path_1 = __importDefault(require("path"));
const utils_1 = require("../utils");
const metadata_1 = require("./metadata");
async function resolveTheme(themePath, variation) {
    const theme = await parseIncludeChain(themePath, variation);
    const approximation = getAlphaApproximation(variation);
    const correctColor = (key) => correctAlpha(approximation, theme.colors[key]);
    return {
        colors: Object.keys(theme.colors).reduce((acc, key) => {
            acc[key] = correctColor(key);
            return acc;
        }, {}),
        settings: theme.settings.map(({ scope, settings: { foreground, background, fontStyle } }) => ({
            scope,
            settings: {
                foreground: foreground ? correctAlpha(approximation, foreground) : foreground,
                background: background ? correctAlpha(approximation, background) : background,
                fontStyle,
            },
        })),
    };
}
exports.resolveTheme = resolveTheme;
async function parseIncludeChain(themePath, variation) {
    let allColors = {};
    let allSettings = [];
    let appliedDefaults = false;
    while (true) {
        const { tokenColors, include, colors } = await parseThemeFile(themePath);
        if (tokenColors != null) {
            // Parent token colors should appear first
            allSettings = [...tokenColors, ...allSettings];
        }
        if (colors != null) {
            // Maintain colors from children
            allColors = { ...colors, ...allColors };
        }
        if (include) {
            themePath = path_1.default.join(path_1.default.dirname(themePath), include);
            continue;
        }
        if (appliedDefaults) {
            break;
        }
        themePath = path_1.default.join(await utils_1.REPO_ROOT, "metadata", "theme-defaults", variation + ".json");
        appliedDefaults = true;
    }
    return {
        colors: allColors,
        settings: allSettings,
    };
}
function getAlphaApproximation(variation) {
    switch (variation) {
        case metadata_1.ThemeVariation.Dark:
        case metadata_1.ThemeVariation.HighContrast:
            // In dark themes, an opaque color is 255, and an invisible one is 0
            return (value, percentage) => value * percentage;
        case metadata_1.ThemeVariation.Light:
            // In light themes, an opaque color is 0, and an invisible one is 255.
            // We calculate its inverse to dim/brighten the correct (inverted) value.
            return (value, percentage) => 255 - (255 - value) * percentage;
    }
}
function correctAlpha(approximation, hexColor) {
    hexColor = hexColor.toUpperCase();
    if (/^#[A-Z0-9]{3}$/.test(hexColor)) {
        const [hash, r, g, b] = hexColor;
        return hash + r + r + g + g + b + b;
    }
    else if (/^#[A-Z0-9]{4}$/.test(hexColor)) {
        const [, r, g, b, a] = hexColor;
        return correctAlphaAux(r + r, g + g, b + b, a + a);
    }
    else if (/^#[A-Z0-9]{6}$/.test(hexColor)) {
        return hexColor;
    }
    else if (/^#[A-Z0-9]{8}$/.test(hexColor)) {
        const [, r1, r2, g1, g2, b1, b2, a1, a2] = hexColor;
        return correctAlphaAux(r1 + r2, g1 + g2, b1 + b2, a1 + a2);
    }
    else {
        throw new Error(`Unsupported color '${hexColor}'.`);
    }
    function correctAlphaAux(redHex, greenHex, blueHex, alphaHex) {
        const red = parseInt(redHex, 16);
        const green = parseInt(greenHex, 16);
        const blue = parseInt(blueHex, 16);
        const alpha = parseInt(alphaHex, 16);
        const percentage = alpha / 256;
        const calc = (value) => Math.round(approximation(value, percentage)).toString(16).toUpperCase().padStart(2, "0");
        return "#" + calc(red) + calc(green) + calc(blue);
    }
}
async function parseThemeFile(filePath) {
    const contents = await fs_extra_1.readFile(filePath, "utf8");
    const errors = [];
    const json = JSONC.parse(contents, errors, {
        allowEmptyContent: false,
        allowTrailingComma: true,
        disallowComments: false,
    });
    assert_1.default(errors.length === 0, `Error parsing: ${filePath}\n` +
        errors
            .map(({ error, offset, length }) => `* [${offset} - ${length}] ${JSONC.printParseErrorCode(error)}`)
            .join("\n"));
    return json;
}
//# sourceMappingURL=theme-resolver.js.map