"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ThemeWrapper = void 0;
const assert_1 = __importDefault(require("assert"));
const path_1 = __importDefault(require("path"));
const TextMate = __importStar(require("vscode-textmate"));
const fs_extra_1 = require("fs-extra");
const metadata_1 = require("./metadata");
const utils_1 = require("../utils");
const utils_2 = require("../utils");
const grammar_wrapper_1 = require("../languages/grammar-wrapper");
const chalk_1 = __importDefault(require("chalk"));
const metadata_2 = require("../languages/metadata");
const theme_resolver_1 = require("./theme-resolver");
const oniguruma_loader_1 = require("./oniguruma-loader");
class ThemeWrapper {
    constructor(theme) {
        this.theme = theme;
        this.userSuppliedGrammars = new Map();
        this.registery = new TextMate.Registry({
            theme: {
                settings: theme ? [...theme.settings] : [],
            },
            onigLib: oniguruma_loader_1.getOnigLib(),
            loadGrammar: async (scopeName) => {
                let grammarFilePath;
                if (this.userSuppliedGrammars.has(scopeName)) {
                    grammarFilePath = this.userSuppliedGrammars.get(scopeName);
                }
                else {
                    const grammars = await utils_1.readAndCacheFile(await utils_2.REPO_ROOT, "metadata", "grammars.json");
                    const metadata = grammars[scopeName];
                    if (!metadata) {
                        return null;
                    }
                    grammarFilePath = path_1.default.join(await utils_2.REPO_ROOT, "metadata", "grammars", metadata.grammarFileName);
                }
                const grammar = await fs_extra_1.readFile(grammarFilePath, "utf8");
                return TextMate.parseRawGrammar(grammar, grammarFilePath);
            },
        });
    }
    static async create(themeIdOrPath) {
        if (!themeIdOrPath) {
            return new ThemeWrapper();
        }
        if (await fs_extra_1.pathExists(themeIdOrPath)) {
            // Assume it is a dark theme. Can we have a better detection here?
            const userSuppliedTheme = await theme_resolver_1.resolveTheme(themeIdOrPath, metadata_1.ThemeVariation.Dark);
            return new ThemeWrapper(userSuppliedTheme);
        }
        const supportedThemePath = path_1.default.join(await utils_2.REPO_ROOT, "metadata", "themes", themeIdOrPath + ".json");
        if (await fs_extra_1.pathExists(supportedThemePath)) {
            const supportedTheme = await fs_extra_1.readJSON(supportedThemePath);
            return new ThemeWrapper(supportedTheme);
        }
        throw new Error("Not a supported theme, or a path to a valid theme file: " + themeIdOrPath);
    }
    async loadGrammar(languageIdOrPath) {
        if (!languageIdOrPath) {
            return new grammar_wrapper_1.GrammarWrapper(
            // Fake tokenizer
            {
                tokenizeLine: (_, ruleStack) => ({ tokens: [], ruleStack }),
                tokenizeLine2: (_, ruleStack) => ({ tokens: new Uint32Array(), ruleStack }),
            }, 
            // empty color map
            []);
        }
        let scopeName;
        if (await fs_extra_1.pathExists(languageIdOrPath)) {
            assert_1.default(path_1.default.extname(languageIdOrPath) === ".json", "Only user supplied JSON textmate grammar files are supported.");
            scopeName = (await fs_extra_1.readJson(languageIdOrPath)).scopeName;
            assert_1.default(scopeName, "User supplied grammar must define a scope name.");
            this.userSuppliedGrammars.set(scopeName, languageIdOrPath);
        }
        else {
            const languages = await metadata_2.getSupportedLanguages();
            const language = languages.find((l) => l.id === languageIdOrPath);
            assert_1.default(language, "Unsupported language: " + languageIdOrPath);
            scopeName = language.scopeName;
        }
        assert_1.default(scopeName, "Unsupported language: " + languageIdOrPath);
        const rawGrammar = await this.registery.loadGrammar(scopeName);
        assert_1.default(rawGrammar, "Unable to resolve grammar file for: " + languageIdOrPath);
        return new grammar_wrapper_1.GrammarWrapper(rawGrammar, this.registery.getColorMap());
    }
    foreground(color) {
        return this.theme ? chalk_1.default.hex(this.theme.colors[color]) : (value) => value;
    }
    background(color) {
        return this.theme ? chalk_1.default.bgHex(this.theme.colors[color]) : (value) => value;
    }
}
exports.ThemeWrapper = ThemeWrapper;
//# sourceMappingURL=theme-wrapper.js.map